# Copyright © 2022 BAAI. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License")
# coding=utf-8
# Copyright (c) 2019, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for using and training tokenizers (char, wordpiece, sentencepiece)"""
from collections import namedtuple
import itertools



import logging
logger = logging.getLogger(__name__)
import os
from flagai.model.file_utils import _get_model_id, _get_vocab_path
from flagai.data.tokenizer.glm_large_ch.glm_large_ch import get_encoder
from flagai.data.tokenizer.glm_10b_en.glm_10b_en_tokenizer import bytes_to_unicode
from flagai.data.tokenizer.glm_large_en.wordpiece import load_vocab, BasicTokenizer, WordpieceTokenizer
import collections
import json
import re


import logging
logger = logging.getLogger(__name__)
import os
from flagai.model.file_utils import _get_model_id, _get_vocab_path
from flagai.data.tokenizer.glm_large_ch.glm_large_ch import get_encoder
from flagai.data.tokenizer.glm_10b_en.glm_10b_en_tokenizer import bytes_to_unicode
from flagai.data.tokenizer.glm_large_en.wordpiece import load_vocab, BasicTokenizer, WordpieceTokenizer
import collections
import json
import re


class BaseTokenizer(object):
    @classmethod
    def from_pretrained(cls,
                        pretrained_model_name_or_path,
                        cache_dir=None,
                        *inputs,
                        **kwargs):
        """
        Instantiate a PreTrainedBertModel from a pre-trained model file.
        Download and cache the pre-trained model file if needed.
        """
        vocab_file = 'vocab.txt'
        merges_file = 'merges.txt'
        sp_file = 'spm.model'
        if cache_dir is None:
            cache_dir = os.path.join(os.path.dirname(__file__), 'vocabs')
        tokenizer_class = "wp"
        # search the cache directory for certain files
        if os.path.exists(cache_dir):
            if os.path.exists(cache_dir + '/' + vocab_file):  # Temporary if statement
                if os.path.exists(cache_dir + '/' + merges_file):  # Temporary if statement
                    tokenizer_class = "bpe"
                else:
                    tokenizer_class = "wp"
            elif os.path.exists(cache_dir + '/' + sp_file):
                tokenizer_class = "sp"
        else:
            model_id = _get_model_id(pretrained_model_name_or_path)
            try:
                _get_vocab_path(cache_dir + '/', vocab_file, model_id, rank=0)
                try:
                    _get_vocab_path(cache_dir + '/', merges_file, model_id, rank=0)
                    tokenizer_class = "bpe"
                except:
                    tokenizer_class = 'wp'
            except:
                try:
                    _get_vocab_path(cache_dir + '/', sp_file, model_id, rank=0)
                    tokenizer_class = "sp"
                except:
                    raise("Error")
        resolved_vocab_file = os.path.join(cache_dir, vocab_file)
        resolved_merges_file = os.path.join(cache_dir, merges_file)
        resolved_sp_file = os.path.join(cache_dir, sp_file)
        if tokenizer_class == "wp":
            return cls._from_pretrained(resolved_vocab_file, tokenizer_class, *inputs, **kwargs)
        elif tokenizer_class == "bpe":
            return cls._from_pretrained(resolved_vocab_file, resolved_merges_file, tokenizer_class, *inputs, **kwargs)
        elif tokenizer_class == "sp":
            return get_encoder(resolved_sp_file, "")

    def __init__(self):
        self.test = 1

    def _from_pretrained(self, vocab_file=None, do_basic_tokenize=True,
                            do_lower_case=True, max_len=None,
                            never_split=("[UNK]", "[SEP]", "[PAD]", "[CLS]", "[MASK]")):
        raise NotImplementedError(
            'TextTokenizer tokens property not implemented')

class WordpieceTokenizer(BaseTokenizer):
    def _from_pretrained(self, vocab_file=None, do_basic_tokenize=True,
                         do_lower_case=True, max_len=None,
                         never_split=("[UNK]", "[SEP]", "[PAD]", "[CLS]", "[MASK]")):
        if not os.path.isfile(vocab_file):
            raise ValueError(
                "Can't find a vocabulary file at path '{}'. To load the vocabulary from a Google pretrained "
                "model use `tokenizer = BertTokenizer.from_pretrained(PRETRAINED_MODEL_NAME)`"
                    .format(vocab_file))
        self.vocab = load_vocab(vocab_file)
        self.ids_to_tokens = collections.OrderedDict([
            (ids, tok) for tok, ids in self.vocab.items()
        ])
        self.do_basic_tokenize = do_basic_tokenize
        if do_basic_tokenize:
            self.basic_tokenizer = BasicTokenizer(do_lower_case=do_lower_case,
                                                  never_split=never_split)
        self.wordpiece_tokenizer = WordpieceTokenizer(vocab=self.vocab)
        self.max_len = max_len if max_len is not None else int(1e12)
        self.tokenizer_class = "wp"

    def __init__(self, name, age):
        self.name = name
        self.age = age














        # if not os.path.isfile(vocab_file):
        #     raise ValueError(
        #         "Can't find a vocabulary file at path '{}'. To load the vocabulary from a Google pretrained "
        #         "model use `tokenizer = BertTokenizer.from_pretrained(PRETRAINED_MODEL_NAME)`"
        #         .format(vocab_file))
        # self.vocab = load_vocab(vocab_file)
        # self.ids_to_tokens = collections.OrderedDict([
        #     (ids, tok) for tok, ids in self.vocab.items()
        # ])
        # self.do_basic_tokenize = do_basic_tokenize
        # if do_basic_tokenize:
        #     self.basic_tokenizer = BasicTokenizer(do_lower_case=do_lower_case,
        #                                           never_split=never_split)
        # self.wordpiece_tokenizer = WordpieceTokenizer(vocab=self.vocab)
        # self.max_len = max_len if max_len is not None else int(1e12)
        # self.tokenizer_class = "wp"
    #
    # def set_special_tokens(self, special_tokens):
    #     """ Add a list of additional tokens to the encoder.
    #         The additional tokens are indexed starting from the last index of the
    #         current vocabulary in the order of the `special_tokens` list.
    #     """
    #     if not special_tokens:
    #         self.special_tokens = {}
    #         self.special_tokens_decoder = {}
    #         return
    #     self.special_tokens = dict((tok, len(self.encoder) + i)
    #                                for i, tok in enumerate(special_tokens))
    #     self.special_tokens_decoder = {
    #         v: k
    #         for k, v in self.special_tokens.items()
    #     }
    #     logger.info("Special tokens {}".format(self.special_tokens))
    #
    # def _from_pretrained_bpe(self,
    #              vocab_file,
    #              merges_file,
    #              errors='replace',
    #              special_tokens=None,
    #              max_len=None):
    #     self.max_len = max_len if max_len is not None else int(1e12)
    #     self.encoder = json.load(open(vocab_file))
    #     self.decoder = {v: k for k, v in self.encoder.items()}
    #     self.errors = errors  # how to handle errors in decoding
    #     self.byte_encoder = bytes_to_unicode()
    #     self.byte_decoder = {v: k for k, v in self.byte_encoder.items()}
    #     bpe_data = open(merges_file, encoding='utf-8').read().split('\n')[1:-1]
    #     bpe_merges = [tuple(merge.split()) for merge in bpe_data]
    #     self.bpe_ranks = dict(zip(bpe_merges, range(len(bpe_merges))))
    #     self.cache = {}
    #
    #     # Should haved added re.IGNORECASE so BPE merges can happen for capitalized versions of contractions
    #     self.pat = re.compile(
    #         r"""'s|'t|'re|'ve|'m|'ll|'d| ?\p{L}+| ?\p{N}+| ?[^\s\p{L}\p{N}]+|\s+(?!\S)|\s+"""
    #     )
    #
    #     self.special_tokens = {}
    #     self.special_tokens_decoder = {}
    #     self.set_special_tokens(special_tokens)
    #     self.tokenizer_class = "bpe"
    #
    #
    # def tokenize(self, text):
    #     if self.do_basic_tokenize:
    #         split_tokens = []
    #         for token in self.basic_tokenizer.tokenize(text):
    #             for sub_token in self.wordpiece_tokenizer.tokenize(token):
    #                 split_tokens.append(sub_token)
    #     else:
    #         split_tokens = self.wordpiece_tokenizer.tokenize(text)
    #     return split_tokens
    #
    # def convert_tokens_to_ids(self, tokens):
    #     """Converts a sequence of tokens into ids using the vocab."""
    #     ids = []
    #     for token in tokens:
    #         ids.append(self.vocab[token])
    #     if len(ids) > self.max_len:
    #         logger.warning(
    #             "Token indices sequence length is longer than the specified maximum "
    #             " sequence length for this BERT model ({} > {}). Running this"
    #             " sequence through BERT will result in indexing errors".format(
    #                 len(ids), self.max_len))
    #     return ids
    #
    # def convert_ids_to_tokens(self, ids):
    #     """Converts a sequence of ids in wordpiece tokens using the vocab."""
    #     tokens = []
    #     for i in ids:
    #         tokens.append(self.ids_to_tokens[i])
    #     return tokens


# from flagai.data.tokenizer.tokenizer import BasicTokenizer


# class BaseTokenizer(object):
#     @classmethod
#     def from_pretrained(cls,
#                         pretrained_model_name_or_path,
#                         cache_dir=None,
#                         *inputs,
#                         **kwargs):
#         """
#         Instantiate a PreTrainedBertModel from a pre-trained model file.
#         Download and cache the pre-trained model file if needed.
#         """
#         vocab_file = 'vocab.txt'
#         merges_file = 'merges.txt'
#         sp_file = 'spm.model'
#         if cache_dir is None:
#             cache_dir = os.path.join(os.path.dirname(__file__), 'vocabs')
#         tokenizer_class = "wp"
#         # search the cache directory for certain files
#         if os.path.exists(cache_dir):
#             if os.path.exists(cache_dir + '/' + vocab_file):  # Temporary if statement
#                 if os.path.exists(cache_dir + '/' + merges_file):  # Temporary if statement
#                     tokenizer_class = "bpe"
#                 else:
#                     tokenizer_class = "wp"
#             elif os.path.exists(cache_dir + '/' + sp_file):
#                 tokenizer_class = "sp"
#         else:
#             model_id = _get_model_id(pretrained_model_name_or_path)
#             try:
#                 _get_vocab_path(cache_dir + '/', vocab_file, model_id, rank=0)
#                 try:
#                     _get_vocab_path(cache_dir + '/', merges_file, model_id, rank=0)
#                     tokenizer_class = "bpe"
#                 except:
#                     tokenizer_class = 'wp'
#             except:
#                 try:
#                     _get_vocab_path(cache_dir + '/', sp_file, model_id, rank=0)
#                     tokenizer_class = "sp"
#                 except:
#                     raise("Error")
#         resolved_vocab_file = os.path.join(cache_dir, vocab_file)
#         resolved_merges_file = os.path.join(cache_dir, merges_file)
#         resolved_sp_file = os.path.join(cache_dir, sp_file)
#         if tokenizer_class == "wp":
#             return cls._from_pretrained_wp(resolved_vocab_file, tokenizer_class, *inputs, **kwargs)
#         elif tokenizer_class == "bpe":
#             return cls._from_pretrained(resolved_vocab_file, resolved_merges_file, tokenizer_class, *inputs, **kwargs)
#         elif tokenizer_class == "sp":
#             return get_encoder(resolved_sp_file, "")
#
#     def _from_pretrained_wp(self, vocab_file=None, do_basic_tokenize=True,
#                             do_lower_case=True, max_len=None,
#                             never_split=("[UNK]", "[SEP]", "[PAD]", "[CLS]", "[MASK]")):
#         if not os.path.isfile(vocab_file):
#             raise ValueError(
#                 "Can't find a vocabulary file at path '{}'. To load the vocabulary from a Google pretrained "
#                 "model use `tokenizer = BertTokenizer.from_pretrained(PRETRAINED_MODEL_NAME)`"
#                 .format(vocab_file))
#         self.vocab = load_vocab(vocab_file)
#         self.ids_to_tokens = collections.OrderedDict([
#             (ids, tok) for tok, ids in self.vocab.items()
#         ])
#         self.do_basic_tokenize = do_basic_tokenize
#         if do_basic_tokenize:
#             self.basic_tokenizer = BasicTokenizer(do_lower_case=do_lower_case,
#                                                   never_split=never_split)
#         self.wordpiece_tokenizer = WordpieceTokenizer(vocab=self.vocab)
#         self.max_len = max_len if max_len is not None else int(1e12)
#         self.tokenizer_class = "wp"
#
#     def set_special_tokens(self, special_tokens):
#         """ Add a list of additional tokens to the encoder.
#             The additional tokens are indexed starting from the last index of the
#             current vocabulary in the order of the `special_tokens` list.
#         """
#         if not special_tokens:
#             self.special_tokens = {}
#             self.special_tokens_decoder = {}
#             return
#         self.special_tokens = dict((tok, len(self.encoder) + i)
#                                    for i, tok in enumerate(special_tokens))
#         self.special_tokens_decoder = {
#             v: k
#             for k, v in self.special_tokens.items()
#         }
#         logger.info("Special tokens {}".format(self.special_tokens))
#
#     def _from_pretrained_bpe(self,
#                  vocab_file,
#                  merges_file,
#                  errors='replace',
#                  special_tokens=None,
#                  max_len=None):
#         self.max_len = max_len if max_len is not None else int(1e12)
#         self.encoder = json.load(open(vocab_file))
#         self.decoder = {v: k for k, v in self.encoder.items()}
#         self.errors = errors  # how to handle errors in decoding
#         self.byte_encoder = bytes_to_unicode()
#         self.byte_decoder = {v: k for k, v in self.byte_encoder.items()}
#         bpe_data = open(merges_file, encoding='utf-8').read().split('\n')[1:-1]
#         bpe_merges = [tuple(merge.split()) for merge in bpe_data]
#         self.bpe_ranks = dict(zip(bpe_merges, range(len(bpe_merges))))
#         self.cache = {}
#
#         # Should haved added re.IGNORECASE so BPE merges can happen for capitalized versions of contractions
#         self.pat = re.compile(
#             r"""'s|'t|'re|'ve|'m|'ll|'d| ?\p{L}+| ?\p{N}+| ?[^\s\p{L}\p{N}]+|\s+(?!\S)|\s+"""
#         )
#
#         self.special_tokens = {}
#         self.special_tokens_decoder = {}
#         self.set_special_tokens(special_tokens)
#         self.tokenizer_class = "bpe"
#
#
#     def tokenize(self, text):
#         if self.do_basic_tokenize:
#             split_tokens = []
#             for token in self.basic_tokenizer.tokenize(text):
#                 for sub_token in self.wordpiece_tokenizer.tokenize(token):
#                     split_tokens.append(sub_token)
#         else:
#             split_tokens = self.wordpiece_tokenizer.tokenize(text)
#         return split_tokens
#
#     def convert_tokens_to_ids(self, tokens):
#         """Converts a sequence of tokens into ids using the vocab."""
#         ids = []
#         for token in tokens:
#             ids.append(self.vocab[token])
#         if len(ids) > self.max_len:
#             logger.warning(
#                 "Token indices sequence length is longer than the specified maximum "
#                 " sequence length for this BERT model ({} > {}). Running this"
#                 " sequence through BERT will result in indexing errors".format(
#                     len(ids), self.max_len))
#         return ids
#
#     def convert_ids_to_tokens(self, ids):
#         """Converts a sequence of ids in wordpiece tokens using the vocab."""
#         tokens = []
#         for i in ids:
#             tokens.append(self.ids_to_tokens[i])
#         return tokens






# Copyright © 2022 BAAI. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License")
# coding=utf-8
# Copyright (c) 2019, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import torch
import math


@torch.jit.script
def gelu_impl(x):
    """OpenAI's gelu implementation."""
    return 0.5 * x * (1.0 + torch.tanh(0.7978845608028654 * x *
                                       (1.0 + 0.044715 * x * x)))


def gelu(x):
    return gelu_impl(x)


def unscaled_init_method(sigma):
    """Init method based on N(0, sigma)."""

    def init_(tensor):
        return torch.nn.init.normal_(tensor, mean=0.0, std=sigma)

    return init_


def scaled_init_method(sigma, num_layers):
    """Init method based on N(0, sigma/sqrt(2*num_layers)."""
    std = sigma / math.sqrt(2.0 * num_layers)

    def init_(tensor):
        return torch.nn.init.normal_(tensor, mean=0.0, std=std)

    return init_


def sqrt(x):
    return int(math.sqrt(x) + 1e-4)



import torch
from flagai.model.blocks.cpm_block import CPM3bmtBlock
from flagai.model.layers.layer_norm_bmt import CPM3bmtLayerNorm
from flagai.model.layers.embeddings_bmt import CPM3bmtEmbedding
from flagai.model.layers.embeddings_bmt import CPM3bmtSegmentPositionEmbedding
from flagai.model.layers.linear_bmt import CPM3bmtLinear

from flagai.model.base_model import BaseModel


import json
import os
import copy
from typing import Any, Dict, Union

import bmtrain as bmt


class CPM3bmtStack(torch.nn.Module):
    """ Layers of encoder transformer blocks plus an final layernorm.

    Args:
        num_layers (int): number of layers.
        dim_model (int): main dimension of modules in transformer blocks.
        dim_ff (int): dim_ff used in :py:class:`model_center.layer.FeedForward`.
        num_heads (int): num_heads used in :py:class:`model_center.layer.Attention`.
        dim_head (int): dim_head used in :py:class:`model_center.layer.Attention`.
        dtype (optional): Defaults to torch.half.
        norm_init_var (float, optional): init_var used in :py:class:`model_center.layer.LayerNorm`. Defaults to 1.0.
        norm_bias (bool, optional): bias used in :py:class:`model_center.layer.LayerNorm`. Defaults to False.
        norm_eps (float, optional): eps used in :py:class:`model_center.layer.LayerNorm`. Defaults to 1e-5.
        att_init_mean (float, optional): init_mean used in :py:class:`model_center.layer.Attention`. Defaults to 0.0.
        att_init_std (float, optional): init_std used in :py:class:`model_center.layer.Attention`. Defaults to 0.02.
        att_bias (bool, optional): bias used in in :py:class:`model_center.layer.Attention`. Defaults to False.
        att_mask_value (float, optional): mask_value used in in :py:class:`model_center.layer.Attention`. Defaults to float("-inf").
        ffn_init_mean (float, optional): init_mean used in :py:class:`model_center.layer.FeedForward`. Defaults to 0.0.
        ffn_init_std (float, optional): init_std used in :py:class:`model_center.layer.FeedForward`. Defaults to 0.02.
        ffn_bias (bool, optional): bias used in :py:class:`model_center.layer.FeedForward`. Defaults to False.
        ffn_activate_fn (str, optional): activate_fn used in :py:class:`model_center.layer.FeedForward`. Defaults to "gated_gelu".
        pos_bias_type (str, optional): pos_bias_type used in :py:class:`model_center.layer.Attention`. Defaults to "none".
        post_layer_norm (bool, optional): whether to use post-layernorm. Defaults to False, which means pre-layernorm.
        attn_scale (bool, optional): attn_scale used in in :py:class:`model_center.layer.Attention`. Defaults to False.
        dropout_p (float, optional): Defaults to 0.
    """
    def __init__(self, config
        ):

        super().__init__()
        
        self.num_layers = config.num_layers

        self.layers = bmt.TransformerBlockList([
            bmt.CheckpointBlock(
                CPM3bmtBlock(config)
            )
            for _ in range(self.num_layers)
        ])

        self.output_layernorm = CPM3bmtLayerNorm(
                    dim_norm = config.dim_model, 
                    bias = config.norm_bias, 
                    dtype = config.dtype,
                    eps = config.norm_eps,
                    init_var = config.norm_init_var)

    def forward(self, hidden_states : torch.Tensor,
                      attention_mask : torch.Tensor,
                      position_bias : torch.Tensor = None,
                      ):
        """
        Args:
            hidden-states (:obj:`torch.Tensor` of shape ``(batch, seq_enc, dim_model)``): Input of encoder, might be the embedding of a batch of sequences. 
            attention_mask (:obj:`torch.Tensor` of shape ``(batch, seq_enc, seq_enc)``): Avoid invalid areas to participate in the calculation 
            position_bias(:obj:`torch.Tensor` of shape ``(num_heads, seq_enc, seq_enc)``) Provides position information to attention mechanism.  

        Return:
            :obj:`torch.Tensor` of shape ``(batch, seq_enc, dim_model)``: The encoder output. 

        """
        # (batch, seq_enc, dim_model)
        hidden_states = self.layers(hidden_states, attention_mask, position_bias, None, None, None)
        # (batch, seq_enc, dim_model)
        hidden_states = self.output_layernorm(hidden_states)
        return hidden_states


class CPM3(BaseModel):
    def __init__(self, config, **kwargs):
        super().__init__(config, **kwargs)
        if type(config) is dict:
                config_cpm3 = CPM3Config(
                num_layers = config['num_layers'],
                dim_model = config['dim_model'], 
                dim_ff = config['dim_ff'],
                num_heads = config['num_heads'],
                dim_head = config['dim_head'],
                int8 = config['int8'],
                norm_eps = config['norm_eps'], 
                norm_init_var = config['norm_init_var'],
                norm_bias = config['norm_bias'],
                att_init_mean = config['att_init_mean'], 
                att_init_std = config['att_init_std'],
                att_bias = config['att_bias'],
                att_mask_value = float(config['att_mask_value']),
                pos_bias_type = config['pos_bias_type'],
                ffn_init_mean = config['ffn_init_mean'], 
                ffn_init_std = config['ffn_init_std'],
                ffn_bias = config['ffn_bias'],
                ffn_activate_fn = config['ffn_activate_fn'],
                length_scale = config['length_scale'],
                attn_scale = config['attn_scale'],
                dropout_p = config['dropout_p'],
                prompt_types = config['prompt_types'],
                prompt_length = config['prompt_length'], 
                emb_init_mean = config['emb_init_mean'],
                proj_init_std = config['proj_init_std'],
                emb_init_std = config['emb_init_std'],
                segment_types = config['segment_types'],
                position_bias_num_buckets = config['position_bias_num_buckets'],
                position_bias_max_distance = config['position_bias_max_distance'], 
                max_exact_rate = config['max_exact_rate'],
                max_distance_rate = config['max_distance_rate'],
                absolute_inner_segment = config['absolute_inner_segment'],
                tied = config['tied'],
            )
        self.config = config_cpm3
        
        self.encoder = CPM3bmtStack(self.config)
        self.prompt_embedding = CPM3bmtEmbedding(
            vocab_size = self.config.prompt_types * self.config.prompt_length, 
            embedding_size = self.config.dim_model,
            length_scale = self.config.length_scale,
            dtype = self.config.dtype,
            int8 = self.config.int8,
            init_mean = self.config.emb_init_mean,
            init_std = self.config.emb_init_std,)

        self.input_embedding = CPM3bmtEmbedding(
            vocab_size = self.config.vocab_size, 
            embedding_size = self.config.dim_model,
            length_scale = self.config.length_scale,
            dtype = self.config.dtype,
            int8 = self.config.int8,
            init_mean = self.config.emb_init_mean,
            init_std = self.config.emb_init_std,)

        self.position_bias = CPM3bmtSegmentPositionEmbedding(
            num_segments = self.config.segment_types,
            num_heads = self.config.num_heads, 
            num_buckets = self.config.position_bias_num_buckets, 
            max_distance = self.config.position_bias_max_distance, 
            max_exact_rate = self.config.max_exact_rate,
            max_distance_rate = self.config.max_distance_rate,
            absolute_inner_segment = self.config.absolute_inner_segment,
            bidirectional = True,
            dtype = self.config.dtype,)
        
        self.prompt_length = self.config.prompt_length
        self.tied = self.config.tied
        self.cls_head = self.config.cls_head
        if self.cls_head:
            self.output_projection = CPM3bmtLinear(
                vocab_size = self.config.cls_head,
                embedding_size = self.config.dim_model,
                length_scale = self.config.length_scale,
                dtype = self.config.dtype,
                int8 = self.config.int8,
                init_mean = self.config.proj_init_mean,
                init_std = self.config.proj_init_std,
                bias = self.config.proj_bias,)
        elif not self.config.tied:
            self.output_projection = CPM3bmtLinear(
                vocab_size = self.config.vocab_size,
                embedding_size = self.config.dim_model,
                length_scale = self.config.length_scale,
                dtype = self.config.dtype,
                int8 = self.config.int8,
                init_mean = self.config.proj_init_mean,
                init_std = self.config.proj_init_std,
                bias = self.config.proj_bias,)

    def forward(self, input : torch.Tensor, # (batch, seqlen)
                      length : torch.Tensor, # (batch)
                      context : torch.Tensor, # (batch, seqlen)
                      position: torch.Tensor, # (batch, seqlen)
                      segment: torch.Tensor, # (batch, seqlen)
                      span : torch.Tensor,  # (batch, seqlen)
                      past_key_values = None,  # num_layers * 2 * (batch, num_heads, seqlen, dim_head)
                      right_ctx_start_idx = None,
                      last_input_idx = None,
                      cached_attn_mask_pos_bias = None,
                      isinfer = True,
                    ):

        batch = input.size(0)
        seqlen = input.size(1)
        input_prompt = input[:, :self.prompt_length].contiguous()
        input_ids = input[:, self.prompt_length:].contiguous()

        prompt_states = self.prompt_embedding(input_prompt)
        hidden_states = self.input_embedding(input_ids)
        
        hidden_states = torch.cat([prompt_states, hidden_states], 1)

        with torch.no_grad():
            device = input.device
            directional_mask_2d = torch.arange(seqlen, device=device) <= torch.arange(seqlen, device=device).view(-1, 1)
            attention_mask = context[:, None, :] | (context[:, :, None].logical_not() & directional_mask_2d.view(1, seqlen, seqlen))
            attention_mask = attention_mask & (span[:, None, :] == span[:, :, None])
            mask_1d = torch.arange(seqlen, device=device)[None, :].repeat(batch, 1) < length[:, None]
            attention_mask = mask_1d.view(batch, seqlen, 1) & mask_1d.view(batch, 1, seqlen) & attention_mask

        position_bias = self.position_bias(position, position, segment, segment)
        hidden_states = self.encoder(hidden_states, attention_mask, position_bias)

        if self.cls_head:
            logits = self.output_projection(hidden_states)
        elif not self.tied:
            logits = self.output_projection(hidden_states)
        else:
            logits = self.input_embedding.projection(hidden_states)

        return logits, hidden_states

        
    
    def load_weights(self, checkpoint_path):
        self.load_state_dict(
            torch.load(checkpoint_path),
            strict = True
        )


class Config(object):

    def __init__(self):
        super().__init__()

    @classmethod
    def from_pretrained(cls, pretrained_model_name_or_path: Union[str, os.PathLike]):
        return cls.from_json_file(pretrained_model_name_or_path)

    @classmethod
    def from_json_file(cls, json_file: Union[str, os.PathLike]):
        config_dict = cls._dict_from_json_file(json_file)
        return cls(**config_dict)

    @classmethod
    def _dict_from_json_file(cls, json_file: Union[str, os.PathLike]):
        with open(json_file, "r", encoding="utf-8") as reader:
            text = reader.read()
        return json.loads(text)

    def to_json_file(self, json_file_path: Union[str, os.PathLike]):
        with open(json_file_path, "w", encoding="utf-8") as writer:
            writer.write(self.to_json_string())

    def to_json_string(self) -> str:
        config_dict = self.to_dict()
        return json.dumps(config_dict, indent=2, sort_keys=True) + "\n"

    def to_dict(self) -> Dict[str, Any]:
        output = copy.deepcopy(self.__dict__)
        if hasattr(self.__class__, "model_type"):
            output["model_type"] = self.__class__.model_type
        return output


class CPM3Config(Config):

    def __init__(self, vocab_size = 30730,
                        dim_model = 4096,
                        num_heads = 64,
                        dim_head = 64,
                        dim_ff = 10240,
                        num_layers = 32,
                        dropout_p = 0.0,
                        emb_init_mean = 0.0,
                        emb_init_std = 1.0,
                        pos_bias_type = "relative",
                        position_bias_num_buckets = 512,
                        position_bias_max_distance = 2048,
                        pos_init_mean = 0.0,
                        pos_init_std = 1.0,
                        norm_init_var = 1.0,
                        norm_bias = False,
                        norm_eps = 1e-6,
                        att_init_mean = 0.0, 
                        att_init_std = 1.0,
                        att_bias = False,
                        att_mask_value = float("-inf"),
                        ffn_init_mean = 0.0, 
                        ffn_init_std = 1.0,
                        ffn_bias = False,
                        ffn_activate_fn = "gated_gelu",
                        proj_init_mean = 0.0,
                        proj_init_std = 1.0,
                        proj_bias = False,
                        length_scale = True,
                        attn_scale = True,
                        half = True, 
                        int8 = False,
                        tied = True,
                        prompt_types = 32,
                        prompt_length = 64, 
                        segment_types = 34,
                        max_exact_rate = 0.25,
                        max_distance_rate = 1.0,
                        absolute_inner_segment = True,
                        cls_head = None,
                        post_layer_norm=False,
                        is_decoder : bool = False,
                        parallel_ffn : bool = False,):

        super().__init__()
        self.prompt_types = prompt_types
        self.prompt_length = prompt_length
        self.segment_types = segment_types
        self.dim_model = dim_model
        self.num_heads = num_heads
        self.dim_head = dim_head
        self.dim_ff = dim_ff
        self.num_layers = num_layers
        self.position_bias_num_buckets = position_bias_num_buckets
        self.position_bias_max_distance = position_bias_max_distance
        self.dropout_p = dropout_p
        self.norm_eps = norm_eps
        self.norm_init_var = norm_init_var
        self.emb_init_mean = emb_init_mean
        self.emb_init_std = emb_init_std
        self.att_init_mean = att_init_mean
        self.att_init_std = att_init_std
        self.ffn_init_mean = ffn_init_mean
        self.ffn_init_std = ffn_init_std
        self.length_scale = length_scale
        self.absolute_inner_segment = absolute_inner_segment
        self.max_distance_rate = max_distance_rate
        self.max_exact_rate = max_exact_rate
        self.int8 = int8
        self.tied = tied
        if half: 
            self.dtype = torch.half
        else:
            self.dtype = torch.float
        self.cls_head = cls_head
        self.vocab_size = vocab_size
        self.pos_bias_type = pos_bias_type
        self.pos_init_mean = pos_init_mean
        self.pos_init_std = pos_init_std
        self.norm_bias = norm_bias
        self.att_bias = att_bias
        self.att_mask_value = att_mask_value
        self.ffn_bias = ffn_bias
        self.ffn_activate_fn = ffn_activate_fn
        self.proj_init_mean = proj_init_mean
        self.proj_init_std = proj_init_std
        self.proj_bias = proj_bias
        self.attn_scale = attn_scale
        self.post_layer_norm = post_layer_norm
        self.is_decoder = is_decoder
        self.parallel_ffn  = parallel_ffn



# Copyright © 2022 BAAI. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License")
from flagai.model.layers.attentions import GPT2Attention
from flagai.model.layers.feedforward import GPT2MLP
from torch import nn


class GPT2Block(nn.Module):

    def __init__(self, n_ctx, config, scale=False):
        super().__init__()
        hidden_size = config.hidden_size
        inner_dim = config.n_inner if config.n_inner is not None else 4 * hidden_size
        self.ln_1 = nn.LayerNorm(hidden_size, eps=config.layer_norm_epsilon)
        self.attn = GPT2Attention(hidden_size, n_ctx, config, scale)
        self.ln_2 = nn.LayerNorm(hidden_size, eps=config.layer_norm_epsilon)
        self.mlp = GPT2MLP(inner_dim, config)
        self.do_layer_norm_before = config.do_layer_norm_before

    def forward(
        self,
        hidden_states,
        layer_past=None,
        attention_mask=None,
        head_mask=None,
        use_cache=False,
        output_attentions=False,
    ):

        residual = hidden_states

        if self.do_layer_norm_before:
            hidden_states = self.ln_1(hidden_states)

        attn_outputs = self.attn(
            hidden_states,
            layer_past=layer_past,
            attention_mask=attention_mask,
            head_mask=head_mask,
            use_cache=use_cache,
            output_attentions=output_attentions,
        )

        attn_output = attn_outputs[0]  # output_attn: a, present, (attentions)
        outputs = attn_outputs[1:]
        # residual connection
        hidden_states = attn_output + residual

        if not self.do_layer_norm_before:
            hidden_states = self.ln_1(hidden_states)

        residual = hidden_states

        if self.do_layer_norm_before:
            hidden_states = self.ln_2(hidden_states)

        feed_forward_hidden_states = self.mlp(hidden_states)
        # residual connection
        hidden_states = residual + feed_forward_hidden_states

        if not self.do_layer_norm_before:
            hidden_states = self.ln_2(hidden_states)

        if use_cache:
            outputs = (hidden_states, ) + outputs
        else:
            outputs = (hidden_states, ) + outputs[1:]

        return outputs  # hidden_states, present, (attentions, cross_attentions)



# Copyright © 2022 BAAI. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License")
# coding=utf-8
# Copyright (c) 2019, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for logging and serialization"""
import os
import random
import time
import numpy as np
import torch
import subprocess
from flagai import mpu
from flagai.logger import log_dist
import torch.distributed as dist


def get_hostname():
    hostname_cmd = ["hostname -I"]
    result = subprocess.check_output(hostname_cmd, shell=True)
    master_addr = result.decode('utf-8').split()[0]
    return master_addr


def get_spare_port(args):
    if torch.distributed.get_rank() == 0:
        port = subprocess.check_output(["shuf -n 1 -i 10000-65535"],
                                       shell=True)
        port = int(port.strip())
        if port == args.master_port:
            port = subprocess.check_output(["shuf -n 1 -i 10000-65535"],
                                           shell=True)
            port = int(port.strip())
        port = torch.cuda.LongTensor([port])
    else:
        port = torch.cuda.LongTensor([0])
    torch.distributed.broadcast(port, 0)
    port = port.item()
    return port


class Timers:
    """Group of timers."""

    class Timer:
        """Timer."""

        def __init__(self, name):
            self.name_ = name
            self.elapsed_ = 0.0
            self.started_ = False
            self.start_time = time.time()

        def start(self):
            """Start the timer."""
            assert not self.started_, 'timer has already been started'
            #torch.cuda.synchronize() #TODO is nessisary
            self.start_time = time.time()
            self.started_ = True

        def stop(self):
            """Stop the timer."""
            assert self.started_, 'timer is not started'
            #torch.cuda.synchronize() #TODO change here
            self.elapsed_ += (time.time() - self.start_time)
            self.started_ = False

        def reset(self):
            """Reset timer."""
            self.elapsed_ = 0.0
            self.started_ = False

        def elapsed(self, reset=True):
            """Calculate the elapsed time."""
            started_ = self.started_
            # If the timing in progress, end it first.
            if self.started_:
                self.stop()
            # Get the elapsed time.
            elapsed_ = self.elapsed_
            # Reset the elapsed time
            if reset:
                self.reset()
            # If timing was in progress, set it back.
            if started_:
                self.start()
            return elapsed_

    def __init__(self):
        self.timers = {}
        self.verbose = True

    def __call__(self, name):
        if name not in self.timers:
            self.timers[name] = self.Timer(name)
        return self.timers[name]

    def log(self, names, normalizer=1.0, reset=True):
        """Log a train_args of timers."""
        assert normalizer > 0.0
        string = 'time (ms)'
        for name in names:
            elapsed_time = self.timers[name].elapsed(
                reset=reset) * 1000.0 / normalizer
            string += ' | {}: {:.2f}'.format(name, elapsed_time)
        if self.verbose:
            log_dist(string, ranks=[0])


def report_memory(name):
    """Simple GPU memory report."""

    mega_bytes = 1024.0 * 1024.0
    string = name + ' memory (MB)'
    string += ' | allocated: {}'.format(torch.cuda.memory_allocated() /
                                        mega_bytes)
    string += ' | max allocated: {}'.format(torch.cuda.max_memory_allocated() /
                                            mega_bytes)
    string += ' | cached: {}'.format(torch.cuda.memory_cached() / mega_bytes)
    string += ' | max cached: {}'.format(torch.cuda.memory_reserved() /
                                         mega_bytes)
    log_dist(string)


def get_checkpoint_name(checkpoints_path, iteration):
    # TODO 根据不同的env_type来设置
    iteration = int(iteration)
    d = '{:d}'.format(iteration)
    env_type = os.getenv("ENV_TYPE")
    if env_type == "deepspeed+mpu":
        filename = 'pytorch_model_{:02d}.bin'.format(
            mpu.get_model_parallel_rank())
    else:
        filename = 'pytorch_model.bin'
    return os.path.join(checkpoints_path, d, filename)


def ensure_directory_exists(filename):
    dirname = os.path.dirname(filename)
    if not os.path.exists(dirname):
        os.makedirs(dirname, exist_ok=True)


def get_checkpoint_tracker_filename(checkpoints_path):
    return os.path.join(checkpoints_path, 'latest_iteration.txt')


def save_checkpoint(iteration,
                    best_iteration,
                    model,
                    optimizer,
                    lr_scheduler,
                    barrier=True,
                    save_dir='checkpoints',
                    only_changed_parameters=False,
                    save_optim=True,
                    save_rng=True):
    """Save a model checkpoint."""
    env_type = os.getenv('ENV_TYPE')
    # Only rank zer0 of the data parallel writes to the disk.
    checkpoint_name = get_checkpoint_name(save_dir, str(iteration))
    log_dist(
        'global rank {} is saving checkpoint at iteration {:7d} to {}'.format(
            0, iteration, checkpoint_name))
    sd = {'iteration': iteration}

    # model state_dict
    while hasattr(model, 'module'): 
        model = model.module
    # if only_changed_parameters:
    #     requires_grad_dict = {}
    #     for name, parameter in model.named_parameters():
    #         requires_grad_dict[name] = parameter.requires_grad
    #     state_dict = {
    #         key: value
    #         for key, value in state_dict.items() if requires_grad_dict[key]
    #     }
    # else:
    state_dict = model.state_dict()
    sd['module'] = state_dict

    # Optimizer stuff.
    if save_optim:
        if optimizer is not None:
            sd['optimizer'] = optimizer.state_dict()
        if lr_scheduler is not None:
            sd['lr_scheduler'] = lr_scheduler.state_dict()
    # rng states.
    if save_rng:
        sd['random_rng_state'] = random.getstate()
        sd['np_rng_state'] = np.random.get_state()
        sd['torch_rng_state'] = torch.get_rng_state()
        sd['cuda_rng_state'] = torch.cuda.get_rng_state()
        sd['rng_tracker_states'] = mpu.get_cuda_rng_tracker().get_states()
    if env_type == 'pytorch' or (env_type != 'deepspeed+mpu'
                                 and dist.get_rank() == 0) or (
                                    env_type == 'deepspeed+mpu'and mpu.get_model_parallel_src_rank() == 0):
        ensure_directory_exists(checkpoint_name)
        config_path = os.path.join(save_dir, str(iteration), 'config.json')

        if hasattr(model, 'save_config'):
            model.save_config(config_path)
            log_dist('  successfully saved {}'.format(config_path))
        torch.save(sd, checkpoint_name)
        log_dist('  successfully saved {}'.format(checkpoint_name))

        tracker_filename = get_checkpoint_tracker_filename(save_dir)
        with open(tracker_filename, 'w') as f:
            f.write(str(iteration) + '\t' + str(best_iteration))

    # Wait so everyone is done (necessary)
    if barrier and dist.is_initialized():
        torch.distributed.barrier()
    # And update the latest iteration


def get_checkpoint_iteration(load_path):
    # Read the tracker file and set the iteration.
    tracker_filename = get_checkpoint_tracker_filename(load_path)
    if not os.path.isfile(tracker_filename):
        log_dist('WARNING: could not find the metadata file {} '.format(
            tracker_filename))
        if os.path.isdir(load_path):
            path = os.path.normpath(load_path)
            load_dir, iteration = os.path.split(path)
            log_dist('Try to directly load the checkpoint from the directory')
            return load_dir, iteration, -1, True
    else:
        log_dist('read the metadata file {} '.format(tracker_filename))
        with open(tracker_filename, 'r', encoding='utf8') as infile:
            iteration, best_iteration = infile.readline().strip().split('\t')
        return load_path, iteration, best_iteration, True
    log_dist('    will not load any checkpoints and will start from '
             'random')
    return load_path, -1, -1, False


def load_checkpoint(model, load_dir="checkpoints", load_type='latest'):
    """Load a model checkpoint."""

    load_dir, iteration, best_iteration, success = get_checkpoint_iteration(
        load_dir)

    if not success:
        return 0

    # Checkpoint.
    if load_type == 'latest':
        checkpoint_name = get_checkpoint_name(load_dir, iteration)
    else:
        checkpoint_name = get_checkpoint_name(load_dir, best_iteration)

    log_dist('global rank {} is loading checkpoint {}'.format(
        0, checkpoint_name))
    sd = torch.load(checkpoint_name, map_location='cpu')

    while hasattr(model, 'module'):
        model = model.module
    model.load_state_dict(sd['module'], strict=False)
    del sd['module']
    return sd


def load_optim(optimizer, lr_scheduler, sd):
    # Optimizer.
    try:
        if optimizer is not None:
            optimizer.load_state_dict(sd['optimizer'])
        if lr_scheduler is not None:
            lr_scheduler.load_state_dict(sd['lr_scheduler'])
        log_dist('global rank 0 is loading optimizer & lr_scheduler')
    except KeyError:
        log_dist('Unable to load optimizer from checkpoint, exiting. '
                 'Specify --no-load-optim or --finetune to prevent '
                 'attempting to load the optimizer '
                 'state.')


def load_rng(sd):
    # rng states.
    env_type = os.getenv('ENV_TYPE')
    try:
        random.setstate(sd['random_rng_state'])
        np.random.set_state(sd['np_rng_state'])
        torch.set_rng_state(sd['torch_rng_state'])
        torch.cuda.set_rng_state(sd['cuda_rng_state'])
        if env_type == 'deepspeed+mpu':
            mpu.get_cuda_rng_tracker().set_states(sd['rng_tracker_states'])
        log_dist('global rank 0 is loading rng states')
    except KeyError:
        log_dist('Unable to load random state from checkpoint, exiting. '
                 'Specify --no-load-rng or --finetune to prevent '
                 'attempting to load the random '
                 'state.')
    log_dist('  successfully loaded rng checkpoints')



# coding=utf-8
# Copyright 2022 The Fairseq Authors and The HuggingFace Inc. team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
""" PyTorch OPT model."""
import random
from typing import List, Optional, Tuple, Union

import torch
import torch.utils.checkpoint
from torch import nn
from torch.nn import CrossEntropyLoss
from flagai.model.layers.activations import ACT2FN
from flagai.model.gpt2_model import GPT2Model, GPT2Stack, GPT2Config
from torch.utils.checkpoint import checkpoint

OPT_PRETRAINED_MODEL_ARCHIVE_LIST = [
    "facebook/opt-125m",
    "facebook/opt-350m",
    "facebook/opt-1.3b",
    "facebook/opt-2.7b",
    "facebook/opt-6.7b",
    "facebook/opt-13b",
    "facebook/opt-30b",
    # See all OPT models at https://huggingface.co/models?filter=opt
]

class OPTLearnedPositionalEmbedding(nn.Embedding):
    """
    This module learns positional embeddings up to a fixed maximum size.
    """

    def __init__(self, num_embeddings: int, embedding_dim: int):
        # OPT is set up so that if padding_idx is specified then offset the embedding ids by 2
        # and adjust num_embeddings appropriately. Other models don't have this hack
        self.offset = 2
        super().__init__(num_embeddings + self.offset, embedding_dim)

    def forward(self, attention_mask: torch.LongTensor, past_key_values_length: int = 0):
        """`input_ids_shape` is expected to be [bsz x seqlen]."""
        attention_mask = attention_mask.long()

        # create positions depending on attention_mask
        positions = (torch.cumsum(attention_mask, dim=1).type_as(attention_mask) * attention_mask).long() - 1
        # cut positions if `past_key_values_length` is > 0
        positions = positions[:, past_key_values_length:]

        return super().forward(positions + self.offset)

class OPTStack(GPT2Stack):
    def __init__(self, config: GPT2Config):
        super(OPTStack, self).__init__(config)
        self.wpe = OPTLearnedPositionalEmbedding(config.n_positions, config.hidden_size)
        self.ln_f = None
        if config.do_layer_norm_before:
            self.ln_f = nn.LayerNorm(config.hidden_size)

        if config.n_embd != config.hidden_size:
            self.project_out = nn.Linear(config.hidden_size, config.n_embd, bias=False)
        else:
            self.project_out = None

        if config.n_embd != config.hidden_size:
            self.project_in = nn.Linear(config.n_embd, config.hidden_size, bias=False)
        else:
            self.project_in = None

    def get_position_embeddings(self, **kwargs):
        pass
        padding_mask = kwargs["padding_mask"]
        past_length = kwargs["past_length"]
        position_embeds = self.wpe(padding_mask, past_length)
        return position_embeds

def trans_opt_to_gpt_config(opt_config_json):
    trans_config_json = {}
    trans_key = {
        "ffn_dim": "n_inner",
        "hidden_size": "hidden_size",
        "max_position_embeddings": "n_positions",
        "num_attention_heads": "n_head",
        "num_hidden_layers": "n_layer",
        "vocab_size": "vocab_size",
        "activation_function": "activation_function",
        "checkpoint_activations": "checkpoint_activations",
        "word_embed_proj_dim": "n_embd",
        "do_layer_norm_before": "do_layer_norm_before",
    }
    for k, v in opt_config_json.items():
        if k in trans_key:
            trans_config_json[trans_key[k]] = v

    return trans_config_json

class OPTModel(GPT2Model):

    def __init__(self, config, **kwargs):
        config = trans_opt_to_gpt_config(config)
        super(OPTModel, self).__init__(config, **kwargs)
        self.transformer = OPTStack(self.config)

    def load_weights(self, checkpoint_path):
        checkpoint = torch.load(checkpoint_path,
                                map_location=torch.device("cpu"))
        if "module" in checkpoint:
            # ddp
            checkpoint = checkpoint["module"]

        checkpoint_ = {}
        for k, v in checkpoint.items():
            if k[:6] == "model.":
                checkpoint_[k[6:]] = v
            else :
                checkpoint_[k] = v

        checkpoint = self.transpose_weight(checkpoint_)
        self.load_state_dict(checkpoint, strict=False)
        self.lm_head.weight.data = nn.Parameter(self.transformer.wte.weight.data)

        return checkpoint

    def transpose_weight(self, checkpoints):

        checkponts_ = {
            "transformer.wte.weight": checkpoints["decoder.embed_tokens.weight"],
            "transformer.wpe.weight": checkpoints["decoder.embed_positions.weight"],
                       }

        if "decoder.project_in.weight" in checkpoints:
            checkponts_["transformer.project_in.weight"] = checkpoints["decoder.project_in.weight"]
            checkponts_["transformer.project_out.weight"] = checkpoints["decoder.project_out.weight"]

        if "decoder.final_layer_norm.weight" in checkpoints:
            checkponts_["transformer.ln_f.weight"] = checkpoints["decoder.final_layer_norm.weight"]
            checkponts_["transformer.ln_f.bias"] = checkpoints["decoder.final_layer_norm.bias"]

        q_weight = None
        k_weight = None
        v_weight = None
        q_bias = None
        k_bias = None
        v_bias = None
        for k, v in checkpoints.items():
            # first ln
            if "decoder.layers" in k and "self_attn_layer_norm" in k:
                layer_id = k.split(".")[2]
                weight_or_bias = k.split(".")[-1]
                checkponts_[f"transformer.h.{layer_id}.ln_1.{weight_or_bias}"] = v
                continue

            # qkv
            if "self_attn.k_proj.weight" in k:
                k_weight = v
                continue
            if "self_attn.k_proj.bias" in k:
                k_bias = v
                continue

            if "self_attn.v_proj.weight" in k:
                v_weight = v
                continue
            if "self_attn.v_proj.bias" in k:
                v_bias = v
                continue

            if "self_attn.q_proj.weight" in k:
                q_weight = v
                qkv_weight = torch.cat([q_weight, k_weight, v_weight], dim=0)
                layer_id = k.split(".")[2]
                checkponts_[f"transformer.h.{layer_id}.attn.c_attn.weight"] = qkv_weight
                continue

            if "self_attn.q_proj.bias" in k:
                q_bias = v
                qkv_bias = torch.cat([q_bias, k_bias, v_bias], dim=0)
                layer_id = k.split(".")[2]
                checkponts_[f"transformer.h.{layer_id}.attn.c_attn.bias"] = qkv_bias
                continue

            # att out
            if "decoder.layers" in k and "self_attn.out_proj" in k:
                layer_id = k.split(".")[2]
                weight_or_bias = k.split(".")[-1]
                checkponts_[f"transformer.h.{layer_id}.attn.c_proj.{weight_or_bias}"] = v
                continue

            # fc1
            if "decoder.layers" in k and "fc1" in k:
                layer_id = k.split(".")[2]
                weight_or_bias = k.split(".")[-1]
                checkponts_[f"transformer.h.{layer_id}.mlp.c_fc.{weight_or_bias}"] = v
                continue

            # fc2
            if "decoder.layers" in k and "fc2" in k:
                layer_id = k.split(".")[2]
                weight_or_bias = k.split(".")[-1]
                checkponts_[f"transformer.h.{layer_id}.mlp.c_proj.{weight_or_bias}"] = v
                continue

            # second ln
            if "decoder.layers" in k and "final_layer_norm" in k:
                layer_id = k.split(".")[2]
                weight_or_bias = k.split(".")[-1]
                checkponts_[f"transformer.h.{layer_id}.ln_2.{weight_or_bias}"] = v
                continue

        return checkponts_


import torch
from flagai.model.layers.attentions import CPM3SelfAttention
from flagai.model.layers.attentions_bmt import CPM3bmtSelfAttention
from flagai.model.layers.attentions import CPM3CrossAttention
from flagai.model.layers.attentions_bmt import CPM3bmtCrossAttention
from flagai.model.layers.feedforward import CPM3FFN
from flagai.model.layers.feedforward_bmt import CPM3bmtFFN
from typing import *


class CPM3Block(torch.nn.Module):
    """ The whole transformer block. A sequence of operation. Consists of self-attention block[, cross-attention block] and feed-forward block.

    Args:
        dim_model (int): main dimension of modules in transformer blocks.
        dim_ff (int): dim_ff used in :py:class:`model_center.layer.FeedForward`.
        num_heads (int): num_heads used in :py:class:`model_center.layer.Attention`.
        dim_head (int): dim_head used in :py:class:`model_center.layer.Attention`.
        is_decoder (bool, optional): whether to use cross-attention. Defaults to False.
        dtype (optional): Defaults to torch.half.
        norm_init_var (float, optional): init_var used in :py:class:`model_center.layer.LayerNorm`. Defaults to 1.0.
        norm_bias (bool, optional): bias used in :py:class:`model_center.layer.LayerNorm`. Defaults to False.
        norm_eps (float, optional): eps used in :py:class:`model_center.layer.LayerNorm`. Defaults to 1e-5.
        att_init_mean (float, optional): init_mean used in :py:class:`model_center.layer.Attention`. Defaults to 0.0.
        att_init_std (float, optional): init_std used in :py:class:`model_center.layer.Attention`. Defaults to 0.02.
        att_bias (bool, optional): bias used in in :py:class:`model_center.layer.Attention`. Defaults to False.
        att_mask_value (float, optional): mask_value used in in :py:class:`model_center.layer.Attention`. Defaults to float("-inf").
        ffn_init_mean (float, optional): init_mean used in :py:class:`model_center.layer.FeedForward`. Defaults to 0.0.
        ffn_init_std (float, optional): init_std used in :py:class:`model_center.layer.FeedForward`. Defaults to 0.02.
        ffn_bias (bool, optional): bias used in :py:class:`model_center.layer.FeedForward`. Defaults to False.
        ffn_activate_fn (str, optional): activate_fn used in :py:class:`model_center.layer.FeedForward`. Defaults to "gated_gelu".
        pos_bias_type (str, optional): pos_bias_type used in :py:class:`model_center.layer.Attention`. Defaults to "none".
        post_layer_norm (bool, optional): whether to use post-layernorm. Defaults to False, which means pre-layernorm.
        attn_scale (bool, optional): attn_scale used in in :py:class:`model_center.layer.Attention`. Defaults to False.
        dropout_p (float, optional): Defaults to 0.
    """

    def __init__(self, config):
        super().__init__()

        self.is_decoder = config.is_decoder

        self.self_att = CPM3SelfAttention(
            dim_model = config.dim_model, 
            num_heads = config.num_heads, 
            dim_head = config.dim_head, 
            dtype = config.dtype,
            int8 = config.int8, 
            norm_eps = config.norm_eps, 
            norm_init_var = config.norm_init_var,
            norm_bias = config.norm_bias,
            att_init_mean = config.att_init_mean, 
            att_init_std = config.att_init_std,
            att_bias = config.att_bias,
            att_mask_value = config.att_mask_value,
            pos_bias_type = config.pos_bias_type,
            post_layer_norm = config.post_layer_norm,
            length_scale = config.length_scale,
            attn_scale = config.attn_scale,
            dropout_p = config.dropout_p,
        )

        if self.is_decoder:
            self.cross_att = CPM3CrossAttention(
                dim_model = config.dim_model, 
                num_heads = config.num_heads, 
                dim_head = config.dim_head, 
                dtype = config.dtype,
                int8 = config.int8, 
                norm_eps = config.norm_eps, 
                norm_init_var = config.norm_init_var,
                norm_bias = config.norm_bias,
                att_init_mean = config.att_init_mean, 
                att_init_std = config.att_init_std,
                att_bias = config.att_bias,
                att_mask_value = config.att_mask_value,
                pos_bias_type = config.pos_bias_type,
                length_scale = config.length_scale,
                attn_scale = config.attn_scale,
                dropout_p = config.dropout_p,
            )
        else:
            self.cross_att = None

        self.ffn = CPM3FFN(
            dim_model = config.dim_model, 
            dim_ff = config.dim_ff,
            dtype = config.dtype, 
            int8 = config.int8,
            norm_eps = config.norm_eps, 
            norm_init_var = config.norm_init_var,
            norm_bias = config.norm_bias,
            ffn_init_mean = config.ffn_init_mean, 
            ffn_init_std = config.ffn_init_std,
            ffn_bias = config.ffn_bias,
            ffn_activate_fn = config.ffn_activate_fn,
            length_scale = config.length_scale,
            dropout_p = config.dropout_p,
            post_layer_norm = config.post_layer_norm,
        )

        self.parallel_ffn = config.parallel_ffn

    def forward(self,
                self_hidden_states : torch.Tensor,
                self_attention_mask : torch.Tensor,
                self_position_bias : Optional[torch.Tensor] = None,
                cross_hidden_states = None,
                cross_attention_mask = None,
                cross_position_bias = None,
                past_key_value = None
            ):
        """
        Args:
            self_hidden_states (:obj:`torch.Tensor` of shape ``(batch, seq_self, dim_model)``): Input of transformer block(self-attention block). It can be the raw embedding of a batch of sequences.
            self_attention_mask (:obj:`torch.Tensor` of shape ``(batch, seq_self, seq_self)``): Avoid invalid areas to participate in the calculation of self-attention.  
            self_position_bias (:obj:`torch.Tensor` of shape ``(num_heads, seq_self, seq_self)``): Provide positional information to self-attention block.
            cross_hidden_states (:obj:`torch.Tensor` of shape ``(batch, seq_cross, dim_model)``): Input of cross-attention block. 
            cross_attention_mask (:obj:`torch.Tensor` of shape ``(batch, seq_self, seq_cross)``): Avoid invalid areas to participate in the calculation of cross-attention.  
            cross_position_bias (:obj:`torch.Tensor` of shape ``(num_heads, seq_self, seq_cross)``): Provide positional information to cross-attention block.

        Return:
            :obj:`torch.Tensor` of shape ``(batch, seq_self, dim_model)``: The output of transformer block.

        """
        # (batch, dim_model, seq_self)
        hidden_states, present_key_value = self.self_att(self_hidden_states,
                                                        attention_mask = self_attention_mask,
                                                        position_bias = self_position_bias,
                                                        past_key_value = past_key_value)

        # (batch, dim_model, seq_self)
        # TODO: use cache in cross_att
        # if self.is_decoder and self.cross_att is not None:
        #     hidden_states = self.cross_att(hidden_states = hidden_states,
        #                                    key_value_states = cross_hidden_states,
        #                                    attention_mask = cross_attention_mask,
        #                                    position_bias = cross_position_bias)

        # (batch, dim_model, seq_self)
        if self.parallel_ffn:
            hidden_states_2 = self.ffn(self_hidden_states)
            hidden_states = hidden_states - self_hidden_states + hidden_states_2
        else:
            hidden_states = self.ffn(hidden_states)

        return hidden_states, present_key_value

class CPM3bmtBlock(torch.nn.Module):
    """ The whole transformer block. A sequence of operation. Consists of self-attention block[, cross-attention block] and feed-forward block.

    Args:
        dim_model (int): main dimension of modules in transformer blocks.
        dim_ff (int): dim_ff used in :py:class:`model_center.layer.FeedForward`.
        num_heads (int): num_heads used in :py:class:`model_center.layer.Attention`.
        dim_head (int): dim_head used in :py:class:`model_center.layer.Attention`.
        is_decoder (bool, optional): whether to use cross-attention. Defaults to False.
        dtype (optional): Defaults to torch.half.
        norm_init_var (float, optional): init_var used in :py:class:`model_center.layer.LayerNorm`. Defaults to 1.0.
        norm_bias (bool, optional): bias used in :py:class:`model_center.layer.LayerNorm`. Defaults to False.
        norm_eps (float, optional): eps used in :py:class:`model_center.layer.LayerNorm`. Defaults to 1e-5.
        att_init_mean (float, optional): init_mean used in :py:class:`model_center.layer.Attention`. Defaults to 0.0.
        att_init_std (float, optional): init_std used in :py:class:`model_center.layer.Attention`. Defaults to 0.02.
        att_bias (bool, optional): bias used in in :py:class:`model_center.layer.Attention`. Defaults to False.
        att_mask_value (float, optional): mask_value used in in :py:class:`model_center.layer.Attention`. Defaults to float("-inf").
        ffn_init_mean (float, optional): init_mean used in :py:class:`model_center.layer.FeedForward`. Defaults to 0.0.
        ffn_init_std (float, optional): init_std used in :py:class:`model_center.layer.FeedForward`. Defaults to 0.02.
        ffn_bias (bool, optional): bias used in :py:class:`model_center.layer.FeedForward`. Defaults to False.
        ffn_activate_fn (str, optional): activate_fn used in :py:class:`model_center.layer.FeedForward`. Defaults to "gated_gelu".
        pos_bias_type (str, optional): pos_bias_type used in :py:class:`model_center.layer.Attention`. Defaults to "none".
        post_layer_norm (bool, optional): whether to use post-layernorm. Defaults to False, which means pre-layernorm.
        attn_scale (bool, optional): attn_scale used in in :py:class:`model_center.layer.Attention`. Defaults to False.
        dropout_p (float, optional): Defaults to 0.
    """

    def __init__(self, config):
        super().__init__()

        self.is_decoder = config.is_decoder

        self.self_att = CPM3bmtSelfAttention(
            dim_model = config.dim_model, 
            num_heads = config.num_heads, 
            dim_head = config.dim_head, 
            dtype = config.dtype,
            int8 = config.int8, 
            norm_eps = config.norm_eps, 
            norm_init_var = config.norm_init_var,
            norm_bias = config.norm_bias,
            att_init_mean = config.att_init_mean, 
            att_init_std = config.att_init_std,
            att_bias = config.att_bias,
            att_mask_value = config.att_mask_value,
            pos_bias_type = config.pos_bias_type,
            post_layer_norm = config.post_layer_norm,
            length_scale = config.length_scale,
            attn_scale = config.attn_scale,
            dropout_p = config.dropout_p,
        )

        if self.is_decoder:
            self.cross_att = CPM3bmtCrossAttention(
                dim_model = config.dim_model, 
                num_heads = config.num_heads, 
                dim_head = config.dim_head, 
                dtype = config.dtype,
                int8 = config.int8, 
                norm_eps = config.norm_eps, 
                norm_init_var = config.norm_init_var,
                norm_bias = config.norm_bias,
                att_init_mean = config.att_init_mean, 
                att_init_std = config.att_init_std,
                att_bias = config.att_bias,
                att_mask_value = config.att_mask_value,
                pos_bias_type = config.pos_bias_type,
                length_scale = config.length_scale,
                attn_scale = config.attn_scale,
                dropout_p = config.dropout_p,
            )
        else:
            self.cross_att = None

        self.ffn = CPM3bmtFFN(
            dim_model = config.dim_model, 
            dim_ff = config.dim_ff,
            dtype = config.dtype, 
            int8 = config.int8,
            norm_eps = config.norm_eps, 
            norm_init_var = config.norm_init_var,
            norm_bias = config.norm_bias,
            ffn_init_mean = config.ffn_init_mean, 
            ffn_init_std = config.ffn_init_std,
            ffn_bias = config.ffn_bias,
            ffn_activate_fn = config.ffn_activate_fn,
            length_scale = config.length_scale,
            dropout_p = config.dropout_p,
            post_layer_norm = config.post_layer_norm,
        )

        self.parallel_ffn = config.parallel_ffn

    def forward(self,
                self_hidden_states : torch.Tensor,
                self_attention_mask : torch.Tensor,
                self_position_bias : Optional[torch.Tensor] = None,
                cross_hidden_states = None,
                cross_attention_mask = None,
                cross_position_bias = None,
            ):
        """
        Args:
            self_hidden_states (:obj:`torch.Tensor` of shape ``(batch, seq_self, dim_model)``): Input of transformer block(self-attention block). It can be the raw embedding of a batch of sequences.
            self_attention_mask (:obj:`torch.Tensor` of shape ``(batch, seq_self, seq_self)``): Avoid invalid areas to participate in the calculation of self-attention.  
            self_position_bias (:obj:`torch.Tensor` of shape ``(num_heads, seq_self, seq_self)``): Provide positional information to self-attention block.
            cross_hidden_states (:obj:`torch.Tensor` of shape ``(batch, seq_cross, dim_model)``): Input of cross-attention block. 
            cross_attention_mask (:obj:`torch.Tensor` of shape ``(batch, seq_self, seq_cross)``): Avoid invalid areas to participate in the calculation of cross-attention.  
            cross_position_bias (:obj:`torch.Tensor` of shape ``(num_heads, seq_self, seq_cross)``): Provide positional information to cross-attention block.

        Return:
            :obj:`torch.Tensor` of shape ``(batch, seq_self, dim_model)``: The output of transformer block.

        """
        # (batch, dim_model, seq_self)
        hidden_states = self.self_att(self_hidden_states,
                                      attention_mask = self_attention_mask,
                                      position_bias = self_position_bias)
        # (batch, dim_model, seq_self)
        if self.is_decoder and self.cross_att is not None:
            hidden_states = self.cross_att(hidden_states = hidden_states,
                                           key_value_states = cross_hidden_states,
                                           attention_mask = cross_attention_mask,
                                           position_bias = cross_position_bias)

        # (batch, dim_model, seq_self)
        if self.parallel_ffn:
            hidden_states_2 = self.ffn(self_hidden_states)
            hidden_states = hidden_states - self_hidden_states + hidden_states_2
        else:
            hidden_states = self.ffn(hidden_states)
        return hidden_states


# Copyright © 2022 BAAI. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License")
# coding=utf-8
# Copyright (c) 2020, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
""" Tasks data utility."""
import copy
import json
import pickle
import re
from typing import Dict, List, Optional
import numpy as np
import random
import torch
from flagai import mpu
try:
    import deepspeed
except:
    pass
import os


def clean_text(text):
    """Remove new lines and multiple spaces and adjust end of sentence dot."""

    text = text.replace("\n", " ")
    text = re.sub(r'\s+', ' ', text)
    for _ in range(3):
        text = text.replace(' . ', '. ')

    return text


class InputExample(object):
    """A raw input example consisting of one or two segments of text and a label"""

    def __init__(self,
                 guid,
                 text_a,
                 text_b=None,
                 label=None,
                 logits=None,
                 meta: Optional[Dict] = None,
                 idx=-1,
                 num_choices=1):
        """
        Create a new InputExample.

        :param guid: a unique textual identifier
        :param text_a: the sequence of text
        :param text_b: an optional, second sequence of text
        :param label: an optional label
        :param logits: an optional list of per-class logits
        :param meta: an optional dictionary to store arbitrary meta information
        :param idx: an optional numeric index
        """
        self.guid = guid
        self.text_a = text_a
        self.text_b = text_b
        self.label = label
        self.logits = logits
        self.idx = idx
        self.num_choices = num_choices
        self.meta = meta if meta else {}

    def __repr__(self):
        return str(self.to_json_string())

    def to_dict(self):
        """Serialize this instance to a Python dictionary."""
        output = copy.deepcopy(self.__dict__)
        return output

    def to_json_string(self):
        """Serialize this instance to a JSON string."""
        return json.dumps(self.to_dict(), indent=2, sort_keys=True) + "\n"

    @staticmethod
    def load_examples(path: str) -> List['InputExample']:
        """Load a set of input examples from a file"""
        with open(path, 'rb') as fh:
            return pickle.load(fh)

    @staticmethod
    def save_examples(examples: List['InputExample'], path: str) -> None:
        """Save a set of input examples to a file"""
        with open(path, 'wb') as fh:
            pickle.dump(examples, fh)


def num_special_tokens_to_add(text_a_ids,
                              text_b_ids,
                              answer_ids,
                              add_cls,
                              add_sep,
                              add_piece,
                              add_eos=True):
    # number of special tokens to add
    num_tokens = 0
    if add_cls:
        num_tokens += 1
    if text_b_ids and add_sep:
        num_tokens += 1
    if add_eos:
        num_tokens += 1
    if not answer_ids and add_piece:
        num_tokens += 1
    return num_tokens


def build_input_from_ids(text_a_ids,
                         text_b_ids,
                         answer_ids,
                         max_seq_length,
                         tokenizer,
                         args=None,
                         add_cls=True,
                         add_sep=False,
                         add_piece=False,
                         add_eos=True,
                         mask_id=None):

    # Prepare ids for special tokens
    if mask_id is None:
        mask_id = tokenizer.get_command_id('MASK')
    eos_id = tokenizer.get_command_id('eos')  # end of sentence token
    cls_id = tokenizer.get_command_id('cls')  # start of sentence token
    sep_id = tokenizer.get_command_id('sep')  # seperator of two texts token

    ids = []  # ids of all the tokens
    types = [
    ]  # types of all the tokens, currently we have 0 for text a, 1 for text b,
    paddings = []  # if-is-padding of all tokens, 1 for no, 0 for yes

    # CLS
    if add_cls:
        ids.append(cls_id)
        types.append(0)  #
        paddings.append(1)

    # add text a
    len_text_a = len(text_a_ids)
    ids.extend(text_a_ids)
    types.extend([0] * len_text_a)
    paddings.extend([1] * len_text_a)

    # add text b
    if text_b_ids is not None:
        # add SEP
        if add_sep:
            ids.append(sep_id)
            types.append(0)
            paddings.append(1)
        len_text_b = len(text_b_ids)
        ids.extend(text_b_ids)
        types.extend([1] * len_text_b)
        paddings.extend([1] * len_text_b)

    eos_length = 1 if add_eos else 0

    # When size exceeds max_seq_length, cut the sequence
    if len(ids) >= max_seq_length - eos_length:
        max_seq_length_m1 = max_seq_length - 1
        ids = ids[0:max_seq_length_m1]
        types = types[0:max_seq_length_m1]
        paddings = paddings[0:max_seq_length_m1]

    # if no text_b, we also should not put 1 at the end
    end_type = 0 if text_b_ids is None else 1
    if add_eos:
        ids.append(eos_id)
        types.append(end_type)
        paddings.append(1)

    sep = len(ids)  # the position where the contents ends
    target_ids = [0] * len(ids)
    loss_masks = [0] * len(ids)
    position_ids = list(range(len(ids)))
    block_position_ids = [0] * len(ids)
    # Piece
    if add_piece or answer_ids is not None:
        sop_id = tokenizer.get_command_id('sop')
        mask_position = ids.index(
            mask_id
        ) if not args.sentinel_token else args.max_position_embeddings
        ids.append(sop_id)
        types.append(end_type)
        paddings.append(1)
        position_ids.append(mask_position)
        block_position_ids.append(1)
        if answer_ids is not None:
            len_answer = len(answer_ids)
            ids.extend(answer_ids[:-1])
            types.extend([end_type] * (len_answer - 1))
            paddings.extend([1] * (len_answer - 1))
            position_ids.extend([mask_position] * (len_answer - 1))
            if not args.no_block_position:
                block_position_ids.extend(range(2, len(answer_ids) + 1))
            else:
                block_position_ids.extend([1] * (len(answer_ids) - 1))
            target_ids.extend(answer_ids)
            loss_masks.extend([1] * len(answer_ids))
        else:
            target_ids.append(0)
            loss_masks.append(1)

    # Padding.
    padding_length = max_seq_length - len(ids)
    if padding_length > 0:
        ids.extend([eos_id] * padding_length)
        types.extend([eos_id] * padding_length)
        paddings.extend([0] * padding_length)
        position_ids.extend([0] * padding_length)
        block_position_ids.extend([0] * padding_length)
        target_ids.extend([0] * padding_length)
        loss_masks.extend([0] * padding_length)
    if not args.masked_lm:
        position_ids = [position_ids, block_position_ids]
    return ids, types, paddings, position_ids, sep, target_ids, loss_masks


#
#
def build_decoder_input(enc_ids, answer_ids, max_seq_length,
                        max_dec_seq_length, tokenizer):
    mask_id = tokenizer.get_command_id('MASK')
    eos_id = tokenizer.get_command_id('eos')
    sop_id = tokenizer.get_command_id('sop')
    masks = []
    # TODO: it probably takes too much memory
    # for i in range(max_dec_seq_length):
    #     m = [1]*enc_len + [0]*(max_seq_length - enc_len) + [1]*(i+1) + [0]*(max_dec_seq_length-1-i)
    #     masks.append(m)
    mask_position = enc_ids.index(mask_id)
    len_answer = len(answer_ids)
    ids = [sop_id] + answer_ids[:-1]
    types = [0] * len_answer  # not used
    paddings = [1] * len_answer
    position_ids = [mask_position] * len_answer
    block_position_ids = list(range(1, len_answer + 1))
    target_ids = answer_ids
    loss_masks = [1] * len_answer
    # Padding.
    padding_length = max_dec_seq_length - len(ids)
    if padding_length > 0:
        ids.extend([eos_id] * padding_length)
        types.extend([0] * padding_length)
        paddings.extend([0] * padding_length)
        position_ids.extend([0] * padding_length)
        block_position_ids.extend([0] * padding_length)
        target_ids.extend([0] * padding_length)
        loss_masks.extend([0] * padding_length)
    position_ids = [position_ids, block_position_ids]
    return ids, types, paddings, position_ids, masks, target_ids, loss_masks


#
#


def build_sample(ids,
                 types=None,
                 paddings=None,
                 positions=None,
                 masks=None,
                 label=None,
                 unique_id=None,
                 target=None,
                 logit_mask=None,
                 segment_ids=None,
                 loss_mask=None,
                 prompt_ids=None,
                 meta=None):
    """Convert to numpy and return a sample consumed by the batch producer."""
    #input_ids = None, position_ids = None, attention_mask = None, target_ids = None, logit_mask = None, prompt_pos = None
    ids_np = np.array(ids, dtype=np.int64)
    sample = {'input_ids': ids_np}
    if label is not None:
        labels_np = np.array(label, dtype=np.int64)
        sample['labels'] = labels_np
    if types is not None:
        types_np = np.array(types, dtype=np.int64)
        sample['types'] = types_np
    if paddings is not None:
        paddings_np = np.array(paddings, dtype=np.int64)
        sample['padding_mask'] = paddings_np
    if positions is not None:
        positions_np = np.array(positions, dtype=np.int64)
        sample['position_ids'] = positions_np
    if masks is not None:
        masks_np = np.array(masks, dtype=np.int64)
        sample['attention_mask'] = masks_np
    if target is not None:
        target_np = np.array(target, dtype=np.int64)
        sample['target_ids'] = target_np
    if logit_mask is not None:
        logit_mask_np = np.array(logit_mask, dtype=np.int64)
        sample['logit_mask'] = logit_mask_np
    if loss_mask is not None:
        loss_mask_np = np.array(loss_mask, dtype=np.int64)
        sample['loss_mask'] = loss_mask_np
    if segment_ids is not None:
        segment_ids = np.array(segment_ids, dtype=np.int64)
        sample['segment_id'] = segment_ids
    if meta is not None:
        sample['meta'] = meta
    if prompt_ids and prompt_ids is not None:
        prompt_ids = np.array(prompt_ids, dtype=np.int64)
        sample['prompt_pos'] = prompt_ids
    if unique_id is not None:
        sample['uid'] = unique_id
    return sample


def build_decoder_sample(sample, dec_ids, dec_position, dec_masks, dec_target,
                         dec_logit_mask):
    sample['dec_text'] = np.array(dec_ids)
    sample['dec_position'] = np.array(dec_position)
    sample['dec_mask'] = np.array(dec_masks)
    sample['dec_target'] = np.array(dec_target)
    sample['dec_logit_mask'] = np.array(dec_logit_mask)
    return sample


def punctuation_standardization(string: str):
    punctuation_dict = {
        "\u201c": "\"",
        "\u201d": "\"",
        "\u2019": "'",
        "\u2018": "'",
        "\u2013": "-"
    }
    for key, value in punctuation_dict.items():
        string = string.replace(key, value)
    return string


def set_deepspeed_activation_checkpointing(args):
    deepspeed.checkpointing.configure(mpu,
                                      deepspeed_config=args.deepspeed_config,
                                      num_checkpoints=args.num_layers)
    mpu.checkpoint = deepspeed.checkpointing.checkpoint
    mpu.get_cuda_rng_tracker = deepspeed.checkpointing.get_cuda_rng_tracker
    mpu.model_parallel_cuda_manual_seed = deepspeed.checkpointing.model_parallel_cuda_manual_seed


def initialize_distributed(args):
    """Initialize torch.distributed."""

    # the automatic assignment of devices has been moved to arguments.py
    torch.cuda.set_device(args.device)
    print("set device")
    # Call the init process
    init_method = 'tcp://'
    # os.environ['MASTER_ADDR'] = '120.92.54.30'
    # os.environ['MASTER_PORT'] = '10501'
    args.master_ip = os.getenv('MASTER_ADDR', 'localhost')
    args.master_port = os.getenv('MASTER_PORT', '6000')
    init_method += args.master_ip + ':' + args.master_port

    torch.distributed.init_process_group(backend=args.distributed_backend,
                                         world_size=args.world_size,
                                         rank=args.rank,
                                         init_method=init_method)

    print("init distributed")
    # Set the model-parallel / data-parallel communicators.
    mpu.initialize_model_parallel(args.model_parallel_size)
    print("set model parallel")
    # Optional DeepSpeed Activation Checkpointing Features
    if hasattr(
            args, "deepspeed"
    ) and args.deepspeed and args.deepspeed_activation_checkpointing:
        set_deepspeed_activation_checkpointing(
            args)  # TODO manual model-parallel seed


def set_random_seed(seed):
    """Set random seed for reproducability."""

    if seed is not None and seed > 0:
        random.seed(seed)
        np.random.seed(seed)
        torch.manual_seed(seed)
        mpu.model_parallel_cuda_manual_seed(seed)
